clear;
clc;
stream=RandStream('mt19937ar');
RandStream.setGlobalStream(stream);

%%parameters
ngrid = [500; 1000; 5000; 10000; 50000; 100000; 500000];
nl = length(ngrid);
kgrid = [2; 3; 4; 5];
kl = length(kgrid);
t = 1;                          %true function
rho = 0.5;                      %correlation between Z and X main simulation
eta = 0.7;                      %correlation between X and e
sigma = 0.5;                    %variance of noise
B = 100;                        %number of simulations is B * rl
rl = 5;
lambda = 0.0;                   %tikhonov penalty
grid = 0.01:0.01:0.99;          %grid to check monotonicity
grid = grid';
G = length(grid);
R_c = zeros(B * rl,nl,kl);
R_u = zeros(B * rl,nl,kl);
R_ratio = zeros(B * rl,nl,kl);
fr_mon = zeros(B * rl,nl,kl);


for ri=1:rl
%%data
zeta_all = randn(ngrid(nl),B);
epsilon_all = randn(ngrid(nl),B);
nu_all = randn(ngrid(nl),B);
xi_all = rho * zeta_all + sqrt(1 - rho^2)*epsilon_all;
Z_all = normcdf(zeta_all);
X_all = normcdf(xi_all);
e_all = sigma * (eta * epsilon_all + sqrt(1 - eta^2) * nu_all);


%%simulations
for ki=1:kl
k = kgrid(ki);
[~,S] = splines(grid,k);
for ni=1:nl
n = ngrid(ni);
bhat_u = zeros(k,B);
bhat_c = zeros(k,B);
for b=1:B
    display([ri k n b]);
    zeta = zeta_all(1:n,b);
    epsilon = epsilon_all(1:n,b);
    nu = nu_all(1:n,b);
    xi = xi_all(1:n,b);
    Z = Z_all(1:n,b);
    X = X_all(1:n,b);
    e = e_all(1:n,b);
    P = splines(X,k);
    Q = splines(Z,k);
    f = myfun(X,t);
    Y = f + e;
    Qzz = Q'*Q/n;
    Qzz1 = inv(Qzz);
    Qxz = P'*Q/n;
    Qzy = Q'*Y/n;
    Qpqy = Qxz*Qzz1*Qzy;
    H = Qxz*Qzz1*Qxz'; H = (H + H')/2;
    bhat_u(:,b) = (H + lambda * eye(k)) \ Qpqy;
    bhat_c(:,b) = quadprog(H,-Qpqy,-S,zeros(G,1));
    R_c(b + (ri - 1) * B, ni, ki) = sum((f - P * bhat_c(:,b)).^2)/n;
    R_u(b + (ri - 1) * B, ni, ki) = sum((f - P * bhat_u(:,b)).^2)/n;
    if min(S * bhat_u(:,b)) >= 0
        fr_mon(b + (ri - 1) * B, ni, ki) = 1;  
    end
end
end
end
clearvars -except R_u R_c ngrid nl kgrid kl t rho eta sigma B rl lambda grid G R_ratio fr_mon;
end

MISE_u = zeros(nl,kl);
MISE_c = zeros(nl,kl);
MISE_ratio = zeros(nl,kl);
R_us = sort(R_u,1); R_cs = sort(R_c,1); R_ratios = sort(R_ratio,1);
for ni=1:nl
    for ki=1:kl
        MISE_u(ni,ki) = R_us(floor(rl * B/2),ni,ki);
        MISE_c(ni,ki) = R_cs(floor(rl * B/2),ni,ki);
        MISE_ratio(ni,ki) = MISE_c(ni,ki) / MISE_u(ni,ki);
    end
end
c = min(MISE_c,[],2);
u = min(MISE_u,[],2);
ratio = c./u;

format long;
display([MISE_c c]);
display([MISE_u u]);
display([MISE_ratio ratio]);



